@@ -74,6 +74,7 @@ typedef struct
 	GHashTable		*vfuncs;		/* string:pointer */
 	GMutex			 vfuncs_mutex;
 	gboolean		 enabled;
+	GType			 app_gtype;
 	gchar			*locale;		/* allow-none */
 	gchar			*language;		/* allow-none */
 	gchar			*name;
@@ -440,6 +441,59 @@ gs_plugin_set_enabled (GsPlugin *plugin, gboolean enabled)
 	priv->enabled = enabled;
 }
 
+/**
+ * gs_plugin_get_app_gtype:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the native application GType.
+ *
+ * Returns: a #GType, e.g. %GS_TYPE_APP
+ *
+ * Since: 3.26
+ **/
+GType
+gs_plugin_get_app_gtype (GsPlugin *plugin)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	return priv->app_gtype;
+}
+
+/**
+ * gs_plugin_set_app_gtype:
+ * @plugin: a #GsPlugin
+ * @app_gtype: the #GType state
+ *
+ * Sets the native application GType.
+ *
+ * Since: 3.26
+ **/
+void
+gs_plugin_set_app_gtype (GsPlugin *plugin, GType app_gtype)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	priv->app_gtype = app_gtype;
+}
+
+/**
+ * gs_plugin_app_new:
+ * @plugin: a #GsPlugin
+ * @id: an application ID
+ *
+ * Creates a GsApp, which may possibly be a subclassed type.
+ * To set the type used when creating objects, plugins must use
+ * gs_plugin_set_app_gtype() with a custom #GType.
+ *
+ * Returns: a #GsApp
+ *
+ * Since: 3.26
+ **/
+GsApp *
+gs_plugin_app_new (GsPlugin *plugin, const gchar *id)
+{
+	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	return GS_APP (g_object_new (priv->app_gtype, "id", id, NULL));
+}
+
 /**
  * gs_plugin_get_name:
  * @plugin: a #GsPlugin
@@ -1664,6 +1718,8 @@ gs_plugin_error_to_string (GsPluginError error)
 		return "restart-required";
 	if (error == GS_PLUGIN_ERROR_AC_POWER_REQUIRED)
 		return "ac-power-required";
+	if (error == GS_PLUGIN_ERROR_TIMED_OUT)
+		return "timed-out";
 	return NULL;
 }
 
@@ -1722,6 +1778,8 @@ gs_plugin_action_to_function_name (GsPluginAction action)
 		return "gs_plugin_add_distro_upgrades";
 	if (action == GS_PLUGIN_ACTION_GET_SOURCES)
 		return "gs_plugin_add_sources";
+	if (action == GS_PLUGIN_ACTION_GET_UNVOTED_REVIEWS)
+		return "gs_plugin_add_unvoted_reviews";
 	if (action == GS_PLUGIN_ACTION_GET_INSTALLED)
 		return "gs_plugin_add_installed";
 	if (action == GS_PLUGIN_ACTION_GET_FEATURED)
@@ -2151,6 +2209,7 @@ gs_plugin_init (GsPlugin *plugin)
 		priv->rules[i] = g_ptr_array_new_with_free_func (g_free);
 
 	priv->enabled = TRUE;
+	priv->app_gtype = GS_TYPE_APP;
 	priv->scale = 1;
 	priv->profile = as_profile_new ();
 	priv->cache = g_hash_table_new_full ((GHashFunc) as_utils_unique_id_hash,
