@@ -88,6 +88,31 @@ gs_os_release_func (void)
 	g_assert_cmpstr (gs_os_release_get_pretty_name (os_release), ==, "Fedora 25 (Workstation Edition)");
 }
 
+static void
+gs_utils_append_kv_func (void)
+{
+	g_autoptr(GString) str = g_string_new (NULL);
+
+	/* normal */
+	gs_utils_append_key_value (str, 5, "key", "val");
+	g_assert_cmpstr (str->str, ==, "key:  val\n");
+
+	/* oversize */
+	g_string_truncate (str, 0);
+	gs_utils_append_key_value (str, 5, "longkey", "val");
+	g_assert_cmpstr (str->str, ==, "longkey: val\n");
+
+	/* null key */
+	g_string_truncate (str, 0);
+	gs_utils_append_key_value (str, 5, NULL, "val");
+	g_assert_cmpstr (str->str, ==, "      val\n");
+
+	/* zero align key */
+	g_string_truncate (str, 0);
+	gs_utils_append_key_value (str, 0, "key", "val");
+	g_assert_cmpstr (str->str, ==, "key: val\n");
+}
+
 static void
 gs_utils_cache_func (void)
 {
@@ -209,14 +234,19 @@ gs_plugin_func (void)
 	GsAppList *list_dup;
 	GsAppList *list_remove;
 	GsApp *app;
-	guint i;
 
 	/* check enums converted */
-	for (i = 0; i < GS_PLUGIN_ACTION_LAST; i++) {
+	for (guint i = 0; i < GS_PLUGIN_ACTION_LAST; i++) {
 		const gchar *tmp = gs_plugin_action_to_string (i);
-		g_assert (tmp != NULL);
+		if (tmp == NULL)
+			g_critical ("failed to convert %u", i);
 		g_assert_cmpint (gs_plugin_action_from_string (tmp), ==, i);
 	}
+	for (guint i = 1; i < GS_PLUGIN_ACTION_LAST; i++) {
+		const gchar *tmp = gs_plugin_action_to_function_name (i);
+		if (tmp == NULL)
+			g_critical ("failed to convert %u", i);
+	}
 
 	/* add a couple of duplicate IDs */
 	app = gs_app_new ("a");
@@ -478,11 +508,11 @@ gs_app_thread_func (void)
 static void
 gs_app_unique_id_func (void)
 {
-	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GsApp) app = gs_app_new (NULL);
 	const gchar *unique_id;
 
 	unique_id = "system/flatpak/gnome/desktop/org.gnome.Software.desktop/master";
-	app = gs_app_new_from_unique_id (unique_id);
+	gs_app_set_from_unique_id (app, unique_id);
 	g_assert (GS_IS_APP (app));
 	g_assert_cmpint (gs_app_get_scope (app), ==, AS_APP_SCOPE_SYSTEM);
 	g_assert_cmpint (gs_app_get_bundle_kind (app), ==, AS_BUNDLE_KIND_FLATPAK);
@@ -612,6 +642,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/gnome-software/lib/utils{wilson}", gs_utils_wilson_func);
 	g_test_add_func ("/gnome-software/lib/utils{error}", gs_utils_error_func);
 	g_test_add_func ("/gnome-software/lib/utils{cache}", gs_utils_cache_func);
+	g_test_add_func ("/gnome-software/lib/utils{append-kv}", gs_utils_append_kv_func);
 	g_test_add_func ("/gnome-software/lib/os-release", gs_os_release_func);
 	g_test_add_func ("/gnome-software/lib/app", gs_app_func);
 	g_test_add_func ("/gnome-software/lib/app{addons}", gs_app_addons_func);
