@@ -33,7 +33,9 @@
 #include <gnome-software.h>
 
 #include "gs-appstream.h"
+#include "gs-flatpak-app.h"
 #include "gs-flatpak.h"
+#include "gs-flatpak-utils.h"
 
 struct GsPluginData {
 	GPtrArray		*flatpaks; /* of GsFlatpak */
@@ -74,6 +76,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 					  g_permission_get_can_acquire (permission);
 	}
 
+	/* unique to us */
+	gs_plugin_set_app_gtype (plugin, GS_TYPE_FLATPAK_APP);
+
 	/* used for self tests */
 	priv->destdir_for_tests = g_getenv ("GS_SELF_TEST_FLATPACK_DATADIR");
 }
@@ -105,7 +110,6 @@ gs_plugin_adopt_app (GsPlugin *plugin, GsApp *app)
 static gboolean
 gs_plugin_flatpak_add_installation (GsPlugin *plugin,
 				    FlatpakInstallation *installation,
-				    GsFlatpakFlags flags,
 				    GCancellable *cancellable,
 				    GError **error)
 {
@@ -118,8 +122,7 @@ gs_plugin_flatpak_add_installation (GsPlugin *plugin,
 	g_assert (ptask != NULL);
 
 	/* create and set up */
-	flatpak = gs_flatpak_new (plugin, installation);
-	gs_flatpak_set_flags (flatpak, flags);
+	flatpak = gs_flatpak_new (plugin, installation, GS_FLATPAK_FLAG_NONE);
 	if (!gs_flatpak_setup (flatpak, cancellable, error))
 		return FALSE;
 	g_debug ("successfully set up %s", gs_flatpak_get_id (flatpak));
@@ -142,13 +145,12 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 		g_autoptr(GPtrArray) installations = NULL;
 		installations = flatpak_get_system_installations (cancellable, error);
 		if (installations == NULL) {
-			gs_plugin_flatpak_error_convert (error);
+			gs_flatpak_error_convert (error);
 			return FALSE;
 		}
 		for (guint i = 0; i < installations->len; i++) {
 			FlatpakInstallation *installation = g_ptr_array_index (installations, i);
 			if (!gs_plugin_flatpak_add_installation (plugin, installation,
-								 GS_FLATPAK_FLAG_NONE,
 								 cancellable, error)) {
 				return FALSE;
 			}
@@ -167,11 +169,10 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 								  cancellable,
 								  error);
 		if (installation == NULL) {
-			gs_plugin_flatpak_error_convert (error);
+			gs_flatpak_error_convert (error);
 			return FALSE;
 		}
 		if (!gs_plugin_flatpak_add_installation (plugin, installation,
-							 GS_FLATPAK_FLAG_NONE,
 							 cancellable, error)) {
 			return FALSE;
 		}
@@ -182,38 +183,10 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 		g_autoptr(FlatpakInstallation) installation = NULL;
 		installation = flatpak_installation_new_user (cancellable, error);
 		if (installation == NULL) {
-			gs_plugin_flatpak_error_convert (error);
+			gs_flatpak_error_convert (error);
 			return FALSE;
 		}
 		if (!gs_plugin_flatpak_add_installation (plugin, installation,
-							 GS_FLATPAK_FLAG_NONE,
-							 cancellable, error)) {
-			return FALSE;
-		}
-	}
-
-	/* add temporary installation for flatpakref files */
-	if (TRUE) {
-		g_autofree gchar *installation_path = NULL;
-		g_autoptr(FlatpakInstallation) installation = NULL;
-		g_autoptr(GFile) installation_file = NULL;
-		installation_path = gs_utils_get_cache_filename ("flatpak",
-								 "installation-tmp",
-								 GS_UTILS_CACHE_FLAG_WRITEABLE,
-								 error);
-		if (installation_path == NULL)
-			return FALSE;
-		installation_file = g_file_new_for_path (installation_path);
-		installation = flatpak_installation_new_for_path (installation_file,
-								  TRUE, /* user */
-								  cancellable,
-								  error);
-		if (installation == NULL) {
-			gs_plugin_flatpak_error_convert (error);
-			return FALSE;
-		}
-		if (!gs_plugin_flatpak_add_installation (plugin, installation,
-							 GS_FLATPAK_FLAG_IS_TEMPORARY,
 							 cancellable, error)) {
 			return FALSE;
 		}
@@ -231,8 +204,6 @@ gs_plugin_add_installed (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_installed (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -248,8 +219,6 @@ gs_plugin_add_sources (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_sources (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -265,8 +234,6 @@ gs_plugin_add_updates (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_updates (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -282,8 +249,6 @@ gs_plugin_add_updates_pending (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_updates_pending (flatpak, list,
 						     cancellable, error)) {
 			return FALSE;
@@ -302,8 +267,6 @@ gs_plugin_refresh (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_refresh (flatpak, cache_age, flags,
 					 cancellable, error)) {
 			return FALSE;
@@ -325,7 +288,7 @@ gs_plugin_flatpak_get_handler (GsPlugin *plugin, GsApp *app)
 	}
 
 	/* specified an explicit name */
-	object_id = gs_app_get_flatpak_object_id (app);
+	object_id = gs_flatpak_app_get_object_id (app);
 	if (object_id != NULL) {
 		for (guint i = 0; i < priv->flatpaks->len; i++) {
 			GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
@@ -373,8 +336,6 @@ gs_plugin_refine_wildcard (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_refine_wildcard (flatpak, app, list, flags,
 						 cancellable, error)) {
 			return FALSE;
@@ -414,18 +375,12 @@ gs_plugin_app_install (GsPlugin *plugin,
 		       GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	GsFlatpak *flatpak = gs_plugin_flatpak_get_handler (plugin, app);
-	if (flatpak == NULL)
-		return TRUE;
+	GsFlatpak *flatpak;
 
-	/* reset the temporary GsFlatpak object ID */
-	if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY) {
+	/* set the app scope */
+	if (gs_app_get_scope (app) == AS_APP_SCOPE_UNKNOWN) {
 		g_autoptr(GSettings) settings = g_settings_new ("org.gnome.software");
 
-		/* no longer valid */
-		g_debug ("resetting temporary object ID");
-		gs_app_set_flatpak_object_id (app, NULL);
-
 		/* get the new GsFlatpak for handling of local files */
 		gs_app_set_scope (app, g_settings_get_boolean (settings, "install-bundles-system-wide") ?
 					AS_APP_SCOPE_SYSTEM : AS_APP_SCOPE_USER);
@@ -437,12 +392,12 @@ gs_plugin_app_install (GsPlugin *plugin,
 			g_debug ("in self tests, using user");
 			gs_app_set_scope (app, AS_APP_SCOPE_USER);
 		}
-		flatpak = gs_plugin_flatpak_get_handler (plugin, app);
-		if (flatpak == NULL)
-			return TRUE;
 	}
 
 	/* actually install */
+	flatpak = gs_plugin_flatpak_get_handler (plugin, app);
+	if (flatpak == NULL)
+		return TRUE;
 	return gs_flatpak_app_install (flatpak, app, cancellable, error);
 }
 
@@ -458,6 +413,223 @@ gs_plugin_update_app (GsPlugin *plugin,
 	return gs_flatpak_update_app (flatpak, app, cancellable, error);
 }
 
+static gboolean
+gs_plugin_flatpak_file_to_app_repo (GsPlugin *plugin,
+				    GsAppList *list,
+				    GFile *file,
+				    GCancellable *cancellable,
+				    GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	g_autoptr(GsApp) app_tmp = NULL;
+	g_autoptr(GsAppList) list_tmp = NULL;
+
+	/* parse the repo file */
+	app_tmp = gs_flatpak_app_new_from_repo_file (file, cancellable, error);
+	if (app_tmp == NULL)
+		return FALSE;
+
+	/* does already exist in either the user or system scope */
+	list_tmp = gs_app_list_new ();
+	for (guint i = 0; i < priv->flatpaks->len; i++) {
+		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
+		if (!gs_flatpak_find_source_by_url (flatpak,
+						    gs_flatpak_app_get_repo_url (app_tmp),
+						    list_tmp, cancellable, error))
+			return FALSE;
+	}
+	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
+		GsApp *app_old = gs_app_list_index (list_tmp, i);
+		if (gs_app_get_state (app_old) == AS_APP_STATE_INSTALLED) {
+			g_debug ("already have %s, using instead of %s",
+				 gs_app_get_unique_id (app_old),
+				 gs_app_get_unique_id (app_tmp));
+			gs_app_list_add (list, app_old);
+			return TRUE;
+		} else {
+			g_warning ("non-installed source %s : %s",
+				   gs_app_get_name (app_old),
+				   as_app_state_to_string (gs_app_get_state (app_old)));
+		}
+	}
+
+	/* this is new */
+	gs_app_list_add (list, app_tmp);
+	gs_app_set_management_plugin (app_tmp, gs_plugin_get_name (plugin));
+	return TRUE;
+}
+
+static GsFlatpak *
+gs_plugin_flatpak_create_temporary (GsPlugin *plugin, GCancellable *cancellable, GError **error)
+{
+	g_autofree gchar *installation_path = NULL;
+	g_autoptr(FlatpakInstallation) installation = NULL;
+	g_autoptr(GFile) installation_file = NULL;
+
+	/* create new per-user installation in a cache dir */
+	installation_path = gs_utils_get_cache_filename ("flatpak",
+							 "installation-tmp",
+							 GS_UTILS_CACHE_FLAG_WRITEABLE |
+							 GS_UTILS_CACHE_FLAG_ENSURE_EMPTY,
+							 error);
+	if (installation_path == NULL)
+		return NULL;
+	installation_file = g_file_new_for_path (installation_path);
+	installation = flatpak_installation_new_for_path (installation_file,
+							  TRUE, /* user */
+							  cancellable,
+							  error);
+	if (installation == NULL) {
+		gs_flatpak_error_convert (error);
+		return NULL;
+	}
+	return gs_flatpak_new (plugin, installation, GS_FLATPAK_FLAG_IS_TEMPORARY);
+}
+
+static gboolean
+gs_plugin_flatpak_file_to_app_bundle (GsPlugin *plugin,
+				      GsAppList *list,
+				      GFile *file,
+				      GCancellable *cancellable,
+				      GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	g_autoptr(GsApp) app_tmp = NULL;
+	g_autoptr(GsAppList) list_tmp = NULL;
+	g_autoptr(GsFlatpak) flatpak_tmp = NULL;
+
+	/* only use the temporary GsFlatpak to avoid the auth dialog */
+	flatpak_tmp = gs_plugin_flatpak_create_temporary (plugin, cancellable, error);
+	if (flatpak_tmp == NULL)
+		return FALSE;
+
+	/* add object */
+	app_tmp = gs_flatpak_file_to_app_bundle (flatpak_tmp, file, cancellable, error);
+	if (app_tmp == NULL)
+		return FALSE;
+
+	/* does already exist in either the user or system scope */
+	list_tmp = gs_app_list_new ();
+	for (guint i = 0; i < priv->flatpaks->len; i++) {
+		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
+		if (!gs_flatpak_find_app (flatpak,
+					  gs_flatpak_app_get_ref_kind (app_tmp),
+					  gs_flatpak_app_get_ref_name (app_tmp),
+					  gs_flatpak_app_get_ref_arch (app_tmp),
+					  gs_flatpak_app_get_ref_branch (app_tmp),
+					  list_tmp, cancellable, error))
+			return FALSE;
+	}
+	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
+		GsApp *app_old = gs_app_list_index (list_tmp, i);
+		if (gs_app_get_state (app_old) == AS_APP_STATE_INSTALLED) {
+			g_debug ("already have %s, using instead of %s",
+				 gs_app_get_unique_id (app_old),
+				 gs_app_get_unique_id (app_tmp));
+			gs_app_list_add (list, app_old);
+			return TRUE;
+		}
+	}
+
+	/* force this to be 'any' scope for installation */
+	gs_app_set_scope (app_tmp, AS_APP_SCOPE_UNKNOWN);
+
+	/* this is new */
+	gs_app_list_add (list, app_tmp);
+	gs_app_set_management_plugin (app_tmp, gs_plugin_get_name (plugin));
+	return TRUE;
+}
+
+static gboolean
+gs_plugin_flatpak_file_to_app_ref (GsPlugin *plugin,
+				   GsAppList *list,
+				   GFile *file,
+				   GCancellable *cancellable,
+				   GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	GsApp *runtime_app;
+	g_autoptr(GsApp) app_tmp = NULL;
+	g_autoptr(GsAppList) list_tmp = NULL;
+	g_autoptr(GsFlatpak) flatpak_tmp = NULL;
+
+	/* only use the temporary GsFlatpak to avoid the auth dialog */
+	flatpak_tmp = gs_plugin_flatpak_create_temporary (plugin, cancellable, error);
+	if (flatpak_tmp == NULL)
+		return FALSE;
+
+	/* add object */
+	app_tmp = gs_flatpak_file_to_app_ref (flatpak_tmp, file, cancellable, error);
+	if (app_tmp == NULL)
+		return FALSE;
+
+	/* does already exist in either the user or system scope */
+	list_tmp = gs_app_list_new ();
+	for (guint i = 0; i < priv->flatpaks->len; i++) {
+		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
+		if (!gs_flatpak_find_app (flatpak,
+					  gs_flatpak_app_get_ref_kind (app_tmp),
+					  gs_flatpak_app_get_ref_name (app_tmp),
+					  gs_flatpak_app_get_ref_arch (app_tmp),
+					  gs_flatpak_app_get_ref_branch (app_tmp),
+					  list_tmp, cancellable, error)) {
+			g_prefix_error (error, "failed to find in existing remotes: ");
+			return FALSE;
+		}
+	}
+	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
+		GsApp *app_old = gs_app_list_index (list_tmp, i);
+		if (gs_app_get_state (app_old) == AS_APP_STATE_INSTALLED) {
+			g_debug ("already have %s, using instead of %s",
+				 gs_app_get_unique_id (app_old),
+				 gs_app_get_unique_id (app_tmp));
+			gs_app_list_add (list, app_old);
+			return TRUE;
+		}
+	}
+
+	/* force this to be 'any' scope for installation */
+	gs_app_set_scope (app_tmp, AS_APP_SCOPE_UNKNOWN);
+
+	/* do we have a system runtime available */
+	runtime_app = gs_app_get_runtime (app_tmp);
+	if (runtime_app != NULL &&
+	    gs_app_get_state (runtime_app) != AS_APP_STATE_INSTALLED) {
+		g_autofree gchar *ref_display = gs_flatpak_app_get_ref_display (runtime_app);
+		g_autoptr(GsAppList) list_runtimes = gs_app_list_new ();
+		for (guint i = 0; i < priv->flatpaks->len; i++) {
+			GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
+			g_debug ("looking for %s in %s",
+				 ref_display, gs_flatpak_get_id (flatpak));
+			if (!gs_flatpak_find_app (flatpak,
+						  gs_flatpak_app_get_ref_kind (runtime_app),
+						  gs_flatpak_app_get_ref_name (runtime_app),
+						  gs_flatpak_app_get_ref_arch (runtime_app),
+						  gs_flatpak_app_get_ref_branch (runtime_app),
+						  list_runtimes,
+						  cancellable, error))
+				return FALSE;
+		}
+		for (guint i = 0; i < gs_app_list_length (list_runtimes); i++) {
+			GsApp *runtime_old = gs_app_list_index (list_runtimes, i);
+			if (gs_app_get_state (runtime_old) == AS_APP_STATE_INSTALLED ||
+			    gs_app_get_state (runtime_old) == AS_APP_STATE_AVAILABLE) {
+				g_debug ("already have %s, using instead of %s",
+					 gs_app_get_unique_id (runtime_old),
+					 gs_app_get_unique_id (runtime_app));
+				gs_app_set_runtime (app_tmp, runtime_old);
+				gs_app_set_update_runtime (app_tmp, runtime_old);
+				break;
+			}
+		}
+	}
+
+	/* this is new */
+	gs_app_list_add (list, app_tmp);
+	gs_app_set_management_plugin (app_tmp, gs_plugin_get_name (plugin));
+	return TRUE;
+}
+
 gboolean
 gs_plugin_file_to_app (GsPlugin *plugin,
 		       GsAppList *list,
@@ -465,43 +637,42 @@ gs_plugin_file_to_app (GsPlugin *plugin,
 		       GCancellable *cancellable,
 		       GError **error)
 {
-	GsPluginData *priv = gs_plugin_get_data (plugin);
-	g_autoptr(GsAppList) list_new = gs_app_list_new ();
+	g_autofree gchar *content_type = NULL;
+	const gchar *mimetypes_bundle[] = {
+		"application/vnd.flatpak",
+		NULL };
+	const gchar *mimetypes_repo[] = {
+		"application/vnd.flatpak.repo",
+		NULL };
+	const gchar *mimetypes_ref[] = {
+		"application/vnd.flatpak.ref",
+		NULL };
 
-	/* only use the temporary GsFlatpak to avoid the auth dialog */
-	for (guint i = 0; i < priv->flatpaks->len; i++) {
-		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY) {
-			if (!gs_flatpak_file_to_app (flatpak, list_new, file,
-						     cancellable, error)) {
-				return FALSE;
-			}
-		}
+	/* does this match any of the mimetypes_bundle we support */
+	content_type = gs_utils_get_content_type (file, cancellable, error);
+	if (content_type == NULL)
+		return FALSE;
+	if (g_strv_contains (mimetypes_bundle, content_type)) {
+		return gs_plugin_flatpak_file_to_app_bundle (plugin,
+							     list,
+							     file,
+							     cancellable,
+							     error);
 	}
-
-	/* force these to be 'any' scope for installation */
-	for (guint i = 0; i < gs_app_list_length (list_new); i++) {
-		GsApp *app_tmp = gs_app_list_index (list_new, i);
-		gs_app_set_scope (app_tmp, AS_APP_SCOPE_UNKNOWN);
+	if (g_strv_contains (mimetypes_repo, content_type)) {
+		return gs_plugin_flatpak_file_to_app_repo (plugin,
+							   list,
+							   file,
+							   cancellable,
+							   error);
 	}
-
-	/* are any of the new list already installed? */
-	for (guint i = 0; i < gs_app_list_length (list_new); i++) {
-		GsApp *app_tmp = gs_app_list_index (list_new, i);
-		g_autoptr(GsApp) app = NULL;
-		app = gs_plugin_cache_lookup (plugin, gs_app_get_unique_id (app_tmp));
-		if (app != NULL) {
-			g_debug ("found existing %s for %s, using",
-				 gs_app_get_unique_id (app),
-				 gs_app_get_unique_id (app_tmp));
-			gs_app_list_add (list, app);
-		} else {
-			g_debug ("no existing %s in plugin cache",
-				 gs_app_get_unique_id (app_tmp));
-			gs_app_list_add (list, app_tmp);
-		}
+	if (g_strv_contains (mimetypes_ref, content_type)) {
+		return gs_plugin_flatpak_file_to_app_ref (plugin,
+							  list,
+							  file,
+							  cancellable,
+							  error);
 	}
-
 	return TRUE;
 }
 
@@ -515,8 +686,6 @@ gs_plugin_add_search (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_search (flatpak, values, list,
 					cancellable, error)) {
 			return FALSE;
@@ -534,8 +703,6 @@ gs_plugin_add_categories (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_categories (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -552,8 +719,6 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_category_apps (flatpak,
 						   category,
 						   list,
@@ -574,8 +739,6 @@ gs_plugin_add_popular (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_popular (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -591,8 +754,6 @@ gs_plugin_add_featured (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_featured (flatpak, list, cancellable, error))
 			return FALSE;
 	}
@@ -609,8 +770,6 @@ gs_plugin_add_recent (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	for (guint i = 0; i < priv->flatpaks->len; i++) {
 		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
-		if (gs_flatpak_get_flags (flatpak) & GS_FLATPAK_FLAG_IS_TEMPORARY)
-			continue;
 		if (!gs_flatpak_add_recent (flatpak, list, age, cancellable, error))
 			return FALSE;
 	}
