@@ -23,8 +23,48 @@
 
 #include "gnome-software-private.h"
 
+#include "gs-flatpak-app.h"
+
 #include "gs-test.h"
 
+static gboolean
+gs_flatpak_test_write_repo_file (const gchar *fn, const gchar *testdir, GError **error)
+{
+	g_autofree gchar *testdir_repourl = NULL;
+	g_autoptr(GString) str = g_string_new (NULL);
+
+	/* create file */
+	testdir_repourl = g_strdup_printf ("file://%s/repo", testdir);
+	g_string_append (str, "[Flatpak Repo]\n");
+	g_string_append (str, "Title=foo-bar\n");
+	g_string_append (str, "Comment=Longer one line comment\n");
+	g_string_append (str, "Description=Longer multiline comment that "
+			      "does into detail.\n");
+	g_string_append (str, "DefaultBranch=master\n");
+	g_string_append_printf (str, "Url=%s\n", testdir_repourl);
+	g_string_append (str, "Homepage=http://foo.bar\n");
+	g_string_append (str, "GPGKey=FOOBAR==\n");
+	return g_file_set_contents (fn, str->str, -1, error);
+}
+
+static gboolean
+gs_flatpak_test_write_ref_file (const gchar *filename, const gchar *url, const gchar *runtimerepo, GError **error)
+{
+	g_autoptr(GString) str = g_string_new (NULL);
+	g_string_append (str, "[Flatpak Ref]\n");
+	g_string_append (str, "Title=Chiron\n");
+	g_string_append (str, "Name=org.test.Chiron\n");
+	g_string_append (str, "Branch=master\n");
+	g_string_append_printf (str, "Url=%s\n", url);
+	g_string_append (str, "IsRuntime=False\n");
+	g_string_append (str, "Comment=Single line synopsis\n");
+	g_string_append (str, "Description=A Testing Application\n");
+	g_string_append (str, "Icon=https://getfedora.org/static/images/fedora-logotext.png\n");
+	if (runtimerepo != NULL)
+		g_string_append_printf (str, "RuntimeRepo=%s\n", runtimerepo);
+	return g_file_set_contents (filename, str->str, -1, error);
+}
+
 static void
 gs_plugins_flatpak_repo_func (GsPluginLoader *plugin_loader)
 {
@@ -42,7 +82,6 @@ gs_plugins_flatpak_repo_func (GsPluginLoader *plugin_loader)
 	g_autoptr(GsApp) app2 = NULL;
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
-	g_autoptr(GString) str = g_string_new (NULL);
 
 	/* no flatpak, abort */
 	if (!gs_plugin_loader_get_enabled (plugin_loader, "flatpak"))
@@ -55,16 +94,7 @@ gs_plugins_flatpak_repo_func (GsPluginLoader *plugin_loader)
 	testdir_repourl = g_strdup_printf ("file://%s/repo", testdir);
 
 	/* create file */
-	g_string_append (str, "[Flatpak Repo]\n");
-	g_string_append (str, "Title=foo-bar\n");
-	g_string_append (str, "Comment=Longer one line comment\n");
-	g_string_append (str, "Description=Longer multiline comment that "
-			      "does into detail.\n");
-	g_string_append (str, "DefaultBranch=stable\n");
-	g_string_append_printf (str, "Url=%s\n", testdir_repourl);
-	g_string_append (str, "Homepage=http://foo.bar\n");
-	g_string_append (str, "GPGKey=FOOBAR==\n");
-	ret = g_file_set_contents (fn, str->str, -1, &error);
+	ret = gs_flatpak_test_write_repo_file (fn, testdir, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -80,7 +110,7 @@ gs_plugins_flatpak_repo_func (GsPluginLoader *plugin_loader)
 	g_assert_no_error (error);
 	g_assert (app != NULL);
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_SOURCE);
-	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE_LOCAL);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "example");
 	g_assert_cmpstr (gs_app_get_management_plugin (app), ==, "flatpak");
 	g_assert_cmpstr (gs_app_get_origin_hostname (app), ==, "localhost");
@@ -200,7 +230,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (kf_remote_repo_version, ==, 1);
 
 	/* add a remote */
-	app_source = gs_app_new ("test");
+	app_source = gs_flatpak_app_new ("test");
 	testdir = gs_test_get_filename (TESTDATADIR, "app-with-runtime");
 	if (testdir == NULL)
 		return;
@@ -208,7 +238,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
 	gs_app_set_management_plugin (app_source, "flatpak");
 	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
-	gs_app_set_metadata (app_source, "flatpak::url", testdir_repourl);
+	gs_flatpak_app_set_repo_url (app_source, testdir_repourl);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
 					 "app", app_source,
 					 NULL);
@@ -289,6 +319,12 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpstr (gs_app_get_update_details (app), ==, NULL);
 	g_assert_cmpint (gs_app_get_update_urgency (app), ==, AS_URGENCY_KIND_UNKNOWN);
 
+	/* check runtime */
+	runtime = gs_app_get_runtime (app);
+	g_assert (runtime != NULL);
+	g_assert_cmpstr (gs_app_get_unique_id (runtime), ==, "user/flatpak/test/runtime/org.test.Runtime/master");
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_AVAILABLE);
+
 	/* install, also installing runtime */
 	g_object_unref (plugin_job);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
@@ -301,6 +337,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
 	g_assert_cmpstr (gs_app_get_version (app), ==, "1.2.3");
 	g_assert_cmpint (gs_app_get_progress (app), ==, 0);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_INSTALLED);
 
 	/* check the application exists in the right places */
 	metadata_fn = g_build_filename (root,
@@ -349,6 +386,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_no_error (error);
 	g_assert (ret);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_INSTALLED);
 	g_assert (!g_file_test (metadata_fn, G_FILE_TEST_IS_REGULAR));
 	g_assert (!g_file_test (desktop_fn, G_FILE_TEST_IS_REGULAR));
 
@@ -378,6 +416,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_no_error (error);
 	g_assert (ret);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_INSTALLED);
 	g_assert (!g_file_test (metadata_fn, G_FILE_TEST_IS_REGULAR));
 	g_assert (!g_file_test (desktop_fn, G_FILE_TEST_IS_REGULAR));
 
@@ -395,9 +434,6 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_state (app_source), ==, AS_APP_STATE_INSTALLED);
 
 	/* remove the runtime */
-	runtime = gs_app_get_runtime (app);
-	g_assert (runtime != NULL);
-	g_assert_cmpstr (gs_app_get_unique_id (runtime), ==, "user/flatpak/test/runtime/org.test.Runtime/master");
 	g_object_unref (plugin_job);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
 					 "app", runtime,
@@ -406,6 +442,7 @@ gs_plugins_flatpak_app_with_runtime_func (GsPluginLoader *plugin_loader)
 	gs_test_flush_main_context ();
 	g_assert_no_error (error);
 	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_AVAILABLE);
 
 	/* remove the remote */
 	g_object_unref (plugin_job);
@@ -448,7 +485,7 @@ gs_plugins_flatpak_app_missing_runtime_func (GsPluginLoader *plugin_loader)
 	}
 
 	/* add a remote */
-	app_source = gs_app_new ("test");
+	app_source = gs_flatpak_app_new ("test");
 	testdir = gs_test_get_filename (TESTDATADIR, "app-missing-runtime");
 	if (testdir == NULL)
 		return;
@@ -456,7 +493,7 @@ gs_plugins_flatpak_app_missing_runtime_func (GsPluginLoader *plugin_loader)
 	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
 	gs_app_set_management_plugin (app_source, "flatpak");
 	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
-	gs_app_set_metadata (app_source, "flatpak::url", testdir_repourl);
+	gs_flatpak_app_set_repo_url (app_source, testdir_repourl);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
 					 "app", app_source,
 					 NULL);
@@ -561,6 +598,17 @@ update_app_action_finish_sync (GObject *source, GAsyncResult *res, gpointer user
 	g_timeout_add_seconds (5, update_app_action_delay_cb, user_data);
 }
 
+static gboolean
+gs_plugins_flatpak_check_app_installing_cb (gpointer user_data)
+{
+	GsApp *app = GS_APP (user_data);
+	if (gs_app_get_state (app) != AS_APP_STATE_INSTALLING) {
+		g_autofree gchar *str = gs_app_to_string (app);
+		g_warning ("expected to be installing: %s", str);
+	}
+	return G_SOURCE_REMOVE;
+}
+
 static void
 gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 {
@@ -573,15 +621,13 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 	g_autofree gchar *testdir2 = NULL;
 	g_autofree gchar *testdir2_repourl = NULL;
 	g_autofree gchar *testdir = NULL;
-	g_autofree gchar *testdir_repourl = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GFile) file = NULL;
+	g_autoptr(GMainLoop) loop = g_main_loop_new (NULL, FALSE);
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GsAppList) sources2 = NULL;
 	g_autoptr(GsAppList) sources = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
-	g_autoptr(GString) str2 = g_string_new (NULL);
-	g_autoptr(GString) str = g_string_new (NULL);
 
 	/* drop all caches */
 	gs_plugin_loader_setup_again (plugin_loader);
@@ -590,13 +636,7 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 	testdir = gs_test_get_filename (TESTDATADIR, "only-runtime");
 	if (testdir == NULL)
 		return;
-	testdir_repourl = g_strdup_printf ("file://%s/repo", testdir);
-	g_string_append (str, "[Flatpak Repo]\n");
-	g_string_append (str, "Title=foo-bar\n");
-	g_string_append (str, "DefaultBranch=master\n");
-	g_string_append_printf (str, "Url=%s\n", testdir_repourl);
-	g_string_append (str, "GPGKey=FOOBAR==\n");
-	ret = g_file_set_contents (fn_repo, str->str, -1, &error);
+	ret = gs_flatpak_test_write_repo_file (fn_repo, testdir, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -606,18 +646,7 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 	if (testdir2 == NULL)
 		return;
 	testdir2_repourl = g_strdup_printf ("file://%s/repo", testdir2);
-	g_string_append (str2, "[Flatpak Ref]\n");
-	g_string_append (str2, "Title=Chiron\n");
-	g_string_append (str2, "Name=org.test.Chiron\n");
-	g_string_append (str2, "Branch=master\n");
-	g_string_append_printf (str2, "Url=%s\n", testdir2_repourl);
-	g_string_append (str2, "IsRuntime=False\n");
-	g_string_append (str2, "Comment=Single line synopsis\n");
-	g_string_append (str2, "Description=A Testing Application\n");
-	g_string_append (str2, "Icon=https://getfedora.org/static/images/fedora-logotext.png\n");
-	g_string_append (str2, "Icon=RuntimeRepo=https://sdk.gnome.org/gnome-nightly.flatpakrepo\n");
-	g_string_append_printf (str2, "RuntimeRepo=%s\n", fn_repourl);
-	ret = g_file_set_contents (fn_ref, str2->str, -1, &error);
+	ret = gs_flatpak_test_write_ref_file (fn_ref, testdir2_repourl, fn_repourl, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -641,8 +670,8 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 
 	/* get runtime */
 	runtime = gs_app_get_runtime (app);
-	g_assert_cmpstr (gs_app_get_unique_id (runtime), ==, "user/flatpak/*/runtime/org.test.Runtime/master");
-	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_UNKNOWN);
+	g_assert_cmpstr (gs_app_get_unique_id (runtime), ==, "user/flatpak/test/runtime/org.test.Runtime/master");
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_AVAILABLE_LOCAL);
 
 	/* check the number of sources */
 	g_object_unref (plugin_job);
@@ -658,10 +687,13 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
 					 "app", app,
 					 NULL);
-	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	g_timeout_add (10, gs_plugins_flatpak_check_app_installing_cb, app);
+	gs_plugin_loader_job_process_async (plugin_loader, plugin_job,
+					    NULL,
+					    update_app_action_finish_sync,
+					    loop);
+	g_main_loop_run (loop);
 	gs_test_flush_main_context ();
-	g_assert_no_error (error);
-	g_assert (ret);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
 	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_INSTALLED);
 
@@ -710,6 +742,248 @@ gs_plugins_flatpak_runtime_repo_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_state (app_source), ==, AS_APP_STATE_AVAILABLE);
 }
 
+/* same as gs_plugins_flatpak_runtime_repo_func, but this time manually
+ * installing the flatpakrepo BEFORE the flatpakref is installed */
+static void
+gs_plugins_flatpak_runtime_repo_redundant_func (GsPluginLoader *plugin_loader)
+{
+	GsApp *app_source;
+	GsApp *runtime;
+	const gchar *fn_ref = "/var/tmp/self-test/test.flatpakref";
+	const gchar *fn_repo = "/var/tmp/self-test/test.flatpakrepo";
+	gboolean ret;
+	g_autofree gchar *fn_repourl = NULL;
+	g_autofree gchar *testdir2 = NULL;
+	g_autofree gchar *testdir2_repourl = NULL;
+	g_autofree gchar *testdir = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GFile) file_repo = NULL;
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GsApp) app_src = NULL;
+	g_autoptr(GsAppList) sources2 = NULL;
+	g_autoptr(GsAppList) sources = NULL;
+	g_autoptr(GsPluginJob) plugin_job = NULL;
+
+	/* drop all caches */
+	gs_plugin_loader_setup_again (plugin_loader);
+
+	/* write a flatpakrepo file */
+	testdir = gs_test_get_filename (TESTDATADIR, "only-runtime");
+	if (testdir == NULL)
+		return;
+	ret = gs_flatpak_test_write_repo_file (fn_repo, testdir, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* convert it to a GsApp */
+	file_repo = g_file_new_for_path (fn_repo);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_FILE_TO_APP,
+					 "file", file_repo,
+					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+					 NULL);
+	app_src = gs_plugin_loader_job_process_app (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (app_src != NULL);
+	g_assert_cmpint (gs_app_get_kind (app_src), ==, AS_APP_KIND_SOURCE);
+	g_assert_cmpint (gs_app_get_state (app_src), ==, AS_APP_STATE_AVAILABLE_LOCAL);
+	g_assert_cmpstr (gs_app_get_id (app_src), ==, "test");
+	g_assert_cmpstr (gs_app_get_unique_id (app_src), ==, "user/*/*/source/test/master");
+	g_assert (gs_app_get_local_file (app_src) != NULL);
+
+	/* install the source manually */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
+					 "app", app_src,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app_src), ==, AS_APP_STATE_INSTALLED);
+
+	/* write a flatpakref file */
+	fn_repourl = g_strdup_printf ("file://%s", fn_repo);
+	testdir2 = gs_test_get_filename (TESTDATADIR, "app-missing-runtime");
+	if (testdir2 == NULL)
+		return;
+	testdir2_repourl = g_strdup_printf ("file://%s/repo", testdir2);
+	ret = gs_flatpak_test_write_ref_file (fn_ref, testdir2_repourl, fn_repourl, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* convert it to a GsApp */
+	file = g_file_new_for_path (fn_ref);
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_FILE_TO_APP,
+					 "file", file,
+					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+					 NULL);
+	app = gs_plugin_loader_job_process_app (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (app != NULL);
+	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE_LOCAL);
+	g_assert_cmpstr (gs_app_get_id (app), ==, "org.test.Chiron.desktop");
+	g_assert (as_utils_unique_id_equal (gs_app_get_unique_id (app),
+			"user/flatpak/org.test.Chiron-origin/desktop/org.test.Chiron.desktop/master"));
+	g_assert (gs_app_get_local_file (app) != NULL);
+
+	/* get runtime */
+	runtime = gs_app_get_runtime (app);
+	g_assert_cmpstr (gs_app_get_unique_id (runtime), ==, "user/flatpak/test/runtime/org.test.Runtime/master");
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_AVAILABLE);
+
+	/* check the number of sources */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_SOURCES, NULL);
+	sources = gs_plugin_loader_job_process (plugin_loader, plugin_job,
+						NULL, &error);
+	g_assert_no_error (error);
+	g_assert (sources != NULL);
+	g_assert_cmpint (gs_app_list_length (sources), ==, 1); /* repo */
+
+	/* install, which will NOT install the runtime from the RuntimeRemote,
+	 * but from the existing test repo */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
+					 "app", app,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_INSTALLED);
+
+	/* check the number of sources */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_SOURCES, NULL);
+	sources2 = gs_plugin_loader_job_process (plugin_loader, plugin_job, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (sources2 != NULL);
+
+	/* remove the app */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
+					 "app", app,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_UNKNOWN);
+
+	/* remove the runtime */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
+					 "app", runtime,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (runtime), ==, AS_APP_STATE_AVAILABLE);
+
+	/* remove the remote */
+	app_source = gs_app_list_index (sources2, 0);
+	g_assert (app_source != NULL);
+	g_assert_cmpstr (gs_app_get_unique_id (app_source), ==, "user/*/*/source/test/*");
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
+					 "app", app_source,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app_source), ==, AS_APP_STATE_AVAILABLE);
+}
+
+static void
+gs_plugins_flatpak_broken_remote_func (GsPluginLoader *plugin_loader)
+{
+	gboolean ret;
+	const gchar *fn = "/tmp/test.flatpakref";
+	g_autofree gchar *testdir2 = NULL;
+	g_autofree gchar *testdir2_repourl = NULL;
+	g_autofree gchar *testdir = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GsApp) app_source = NULL;
+	g_autoptr(GsPluginJob) plugin_job = NULL;
+
+	/* drop all caches */
+	gs_plugin_loader_setup_again (plugin_loader);
+
+	/* no flatpak, abort */
+	if (!gs_plugin_loader_get_enabled (plugin_loader, "flatpak"))
+		return;
+
+	/* add a remote with only the runtime in */
+	app_source = gs_flatpak_app_new ("test");
+	testdir = gs_test_get_filename (TESTDATADIR, "only-runtime");
+	if (testdir == NULL)
+		return;
+	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
+	gs_app_set_management_plugin (app_source, "flatpak");
+	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
+	gs_flatpak_app_set_repo_url (app_source, "file:///wont/work");
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
+					 "app", app_source,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app_source), ==, AS_APP_STATE_INSTALLED);
+
+	/* write a flatpakref file */
+	testdir2 = gs_test_get_filename (TESTDATADIR, "app-with-runtime");
+	if (testdir2 == NULL)
+		return;
+	testdir2_repourl = g_strdup_printf ("file://%s/repo", testdir2);
+	ret = gs_flatpak_test_write_ref_file (fn, testdir2_repourl, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* convert it to a GsApp */
+	file = g_file_new_for_path (fn);
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_FILE_TO_APP,
+					 "file", file,
+					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+					 NULL);
+	app = gs_plugin_loader_job_process_app (plugin_loader, plugin_job, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (app != NULL);
+	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE_LOCAL);
+	g_assert_cmpstr (gs_app_get_id (app), ==, "org.test.Chiron.desktop");
+	g_assert (as_utils_unique_id_equal (gs_app_get_unique_id (app),
+			"user/flatpak/org.test.Chiron-origin/desktop/org.test.Chiron.desktop/master"));
+	g_assert_cmpstr (gs_app_get_url (app, AS_URL_KIND_HOMEPAGE), ==, "http://127.0.0.1/");
+	g_assert_cmpstr (gs_app_get_name (app), ==, "Chiron");
+	g_assert_cmpstr (gs_app_get_summary (app), ==, "Single line synopsis");
+	g_assert_cmpstr (gs_app_get_description (app), ==, "Long description.");
+	g_assert_cmpstr (gs_app_get_version (app), ==, "1.2.3");
+	g_assert (gs_app_get_local_file (app) != NULL);
+
+	/* remove source */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
+					 "app", app_source,
+					 NULL);
+	ret = gs_plugin_loader_job_action (plugin_loader, plugin_job, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+}
+
 static void
 gs_plugins_flatpak_ref_func (GsPluginLoader *plugin_loader)
 {
@@ -740,7 +1014,7 @@ gs_plugins_flatpak_ref_func (GsPluginLoader *plugin_loader)
 		return;
 
 	/* add a remote with only the runtime in */
-	app_source = gs_app_new ("test");
+	app_source = gs_flatpak_app_new ("test");
 	testdir = gs_test_get_filename (TESTDATADIR, "only-runtime");
 	if (testdir == NULL)
 		return;
@@ -748,7 +1022,7 @@ gs_plugins_flatpak_ref_func (GsPluginLoader *plugin_loader)
 	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
 	gs_app_set_management_plugin (app_source, "flatpak");
 	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
-	gs_app_set_metadata (app_source, "flatpak::url", testdir_repourl);
+	gs_flatpak_app_set_repo_url (app_source, testdir_repourl);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
 					 "app", app_source,
 					 NULL);
@@ -799,16 +1073,7 @@ gs_plugins_flatpak_ref_func (GsPluginLoader *plugin_loader)
 	if (testdir2 == NULL)
 		return;
 	testdir2_repourl = g_strdup_printf ("file://%s/repo", testdir2);
-	g_string_append (str, "[Flatpak Ref]\n");
-	g_string_append (str, "Title=Chiron\n");
-	g_string_append (str, "Name=org.test.Chiron\n");
-	g_string_append (str, "Branch=master\n");
-	g_string_append_printf (str, "Url=%s\n", testdir2_repourl);
-	g_string_append (str, "IsRuntime=False\n");
-	g_string_append (str, "Comment=Single line synopsis\n");
-	g_string_append (str, "Description=A Testing Application\n");
-	g_string_append (str, "Icon=https://getfedora.org/static/images/fedora-logotext.png\n");
-	ret = g_file_set_contents (fn, str->str, -1, &error);
+	ret = gs_flatpak_test_write_ref_file (fn, testdir2_repourl, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
@@ -867,6 +1132,20 @@ gs_plugins_flatpak_ref_func (GsPluginLoader *plugin_loader)
 	app_tmp = gs_app_list_index (search1, 0);
 	g_assert_cmpstr (gs_app_get_id (app_tmp), ==, "org.test.Chiron.desktop");
 
+	/* convert it to a GsApp again, and get the installed thing */
+	g_object_unref (plugin_job);
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_FILE_TO_APP,
+					 "file", file,
+					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+					 NULL);
+	app = gs_plugin_loader_job_process_app (plugin_loader, plugin_job, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (app != NULL);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
+	g_assert (as_utils_unique_id_equal (gs_app_get_unique_id (app),
+		  "user/flatpak/org.test.Chiron-origin/desktop/org.test.Chiron.desktop/master"));
+
 	/* remove app */
 	g_object_unref (plugin_job);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REMOVE,
@@ -971,11 +1250,11 @@ gs_plugins_flatpak_app_update_func (GsPluginLoader *plugin_loader)
 	g_assert (symlink (repodir1_fn, "/var/tmp/self-test/repo") == 0);
 
 	/* add a remote */
-	app_source = gs_app_new ("test");
+	app_source = gs_flatpak_app_new ("test");
 	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
 	gs_app_set_management_plugin (app_source, "flatpak");
 	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
-	gs_app_set_metadata (app_source, "flatpak::url", "file:///var/tmp/self-test/repo");
+	gs_flatpak_app_set_repo_url (app_source, "file:///var/tmp/self-test/repo");
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_INSTALL,
 					 "app", app_source,
 					 NULL);
@@ -1214,9 +1493,15 @@ main (int argc, char **argv)
 	g_test_add_data_func ("/gnome-software/plugins/flatpak/ref",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugins_flatpak_ref_func);
+	g_test_add_data_func ("/gnome-software/plugins/flatpak/broken-remote",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugins_flatpak_broken_remote_func);
 	g_test_add_data_func ("/gnome-software/plugins/flatpak/runtime-repo",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugins_flatpak_runtime_repo_func);
+	g_test_add_data_func ("/gnome-software/plugins/flatpak/runtime-repo-redundant",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugins_flatpak_runtime_repo_redundant_func);
 	g_test_add_data_func ("/gnome-software/plugins/flatpak/app-update-runtime",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugins_flatpak_app_update_func);
