@@ -21,26 +21,33 @@
 
 #include <config.h>
 
+#include <gio/gdesktopappinfo.h>
 #include <snapd-glib/snapd-glib.h>
 #include <gnome-software.h>
 
 struct GsPluginData {
-	GsAuth		*auth;
-	GHashTable	*store_snaps;
+	SnapdClient		*client;
+	SnapdSystemConfinement	 system_confinement;
+	GsAuth			*auth;
+	GHashTable		*store_snaps;
 };
 
 void
 gs_plugin_initialize (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_alloc_data (plugin, sizeof(GsPluginData));
-	g_autoptr(SnapdClient) client = NULL;
+	const gchar *old_user_agent;
+	g_autofree gchar *user_agent = NULL;
 	g_autoptr (GError) error = NULL;
 
-	client = snapd_client_new ();
-	if (!snapd_client_connect_sync (client, NULL, &error)) {
+	priv->client = snapd_client_new ();
+	if (!snapd_client_connect_sync (priv->client, NULL, &error)) {
 		gs_plugin_set_enabled (plugin, FALSE);
 		return;
 	}
+	old_user_agent = snapd_client_get_user_agent (priv->client);
+	user_agent = g_strdup_printf ("%s %s", gs_user_agent (), old_user_agent);
+	snapd_client_set_user_agent (priv->client, user_agent);
 
 	priv->store_snaps = g_hash_table_new_full (g_str_hash, g_str_equal,
 						   g_free, (GDestroyNotify) g_object_unref);
@@ -59,10 +66,105 @@ gs_plugin_initialize (GsPlugin *plugin)
 	gs_plugin_set_appstream_id (plugin, "org.gnome.Software.Plugin.Snap");
 }
 
+static void
+snapd_error_convert (GError **perror)
+{
+	GError *error = perror != NULL ? *perror : NULL;
+
+	/* not set */
+	if (error == NULL)
+		return;
+
+	/* this are allowed for low-level errors */
+	if (gs_utils_error_convert_gio (perror))
+		return;
+
+	/* custom to this plugin */
+	if (error->domain == SNAPD_ERROR) {
+		switch (error->code) {
+		case SNAPD_ERROR_AUTH_DATA_REQUIRED:
+			error->code = GS_PLUGIN_ERROR_AUTH_REQUIRED;
+			g_free (error->message);
+			error->message = g_strdup ("Requires authentication with @snapd");
+			break;
+		case SNAPD_ERROR_TWO_FACTOR_REQUIRED:
+			error->code = GS_PLUGIN_ERROR_PIN_REQUIRED;
+			break;
+		case SNAPD_ERROR_AUTH_DATA_INVALID:
+		case SNAPD_ERROR_TWO_FACTOR_INVALID:
+			error->code = GS_PLUGIN_ERROR_AUTH_INVALID;
+			break;
+		case SNAPD_ERROR_CONNECTION_FAILED:
+		case SNAPD_ERROR_WRITE_FAILED:
+		case SNAPD_ERROR_READ_FAILED:
+		case SNAPD_ERROR_BAD_REQUEST:
+		case SNAPD_ERROR_BAD_RESPONSE:
+		case SNAPD_ERROR_PERMISSION_DENIED:
+		case SNAPD_ERROR_FAILED:
+		case SNAPD_ERROR_TERMS_NOT_ACCEPTED:
+		case SNAPD_ERROR_PAYMENT_NOT_SETUP:
+		case SNAPD_ERROR_PAYMENT_DECLINED:
+		case SNAPD_ERROR_ALREADY_INSTALLED:
+		case SNAPD_ERROR_NOT_INSTALLED:
+		case SNAPD_ERROR_NO_UPDATE_AVAILABLE:
+		case SNAPD_ERROR_PASSWORD_POLICY_ERROR:
+		case SNAPD_ERROR_NEEDS_DEVMODE:
+		case SNAPD_ERROR_NEEDS_CLASSIC:
+		case SNAPD_ERROR_NEEDS_CLASSIC_SYSTEM:
+		default:
+			error->code = GS_PLUGIN_ERROR_FAILED;
+			break;
+		}
+	} else {
+		g_warning ("can't reliably fixup error from domain %s",
+			   g_quark_to_string (error->domain));
+		error->code = GS_PLUGIN_ERROR_FAILED;
+	}
+	error->domain = GS_PLUGIN_ERROR;
+}
+
+static void
+load_auth (GsPlugin *plugin)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	GsAuth *auth;
+	const gchar *serialized_macaroon;
+	g_autoptr(GVariant) macaroon_variant = NULL;
+	const gchar *macaroon;
+	g_auto(GStrv) discharges = NULL;
+	g_autoptr(SnapdAuthData) auth_data = NULL;
+
+	auth = gs_plugin_get_auth_by_id (plugin, "snapd");
+	if (auth == NULL)
+		return;
+
+	serialized_macaroon = gs_auth_get_metadata_item (auth, "macaroon");
+	if (serialized_macaroon == NULL)
+		return;
+
+	macaroon_variant = g_variant_parse (G_VARIANT_TYPE ("(sas)"),
+					    serialized_macaroon,
+					    NULL,
+					    NULL,
+					    NULL);
+	if (macaroon_variant == NULL)
+		return;
+
+	g_variant_get (macaroon_variant, "(&s^as)", &macaroon, &discharges);
+	auth_data = snapd_auth_data_new (macaroon, discharges);
+	snapd_client_set_auth_data (priv->client, auth_data);
+}
+
 gboolean
 gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
+	g_autoptr(SnapdSystemInformation) system_information = NULL;
+
+	system_information = snapd_client_get_system_information_sync (priv->client, cancellable, error);
+	if (system_information == NULL)
+		return FALSE;
+	priv->system_confinement = snapd_system_information_get_confinement (system_information);
 
 	/* load from disk */
 	gs_auth_add_metadata (priv->auth, "macaroon", NULL);
@@ -71,54 +173,12 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 				 GS_AUTH_STORE_FLAG_METADATA,
 				 cancellable, error))
 		return FALSE;
+	load_auth (plugin);
 
 	/* success */
 	return TRUE;
 }
 
-static SnapdAuthData *
-get_auth (GsPlugin *plugin)
-{
-	GsAuth *auth;
-	const gchar *serialized_macaroon;
-	g_autoptr(GVariant) macaroon_variant = NULL;
-	const gchar *macaroon;
-	g_auto(GStrv) discharges = NULL;
-	g_autoptr (GError) error_local = NULL;
-
-	auth = gs_plugin_get_auth_by_id (plugin, "snapd");
-	if (auth == NULL)
-		return NULL;
-	serialized_macaroon = gs_auth_get_metadata_item (auth, "macaroon");
-	if (serialized_macaroon == NULL)
-		return NULL;
-	macaroon_variant = g_variant_parse (G_VARIANT_TYPE ("(sas)"),
-					    serialized_macaroon,
-					    NULL,
-					    NULL,
-					    NULL);
-	if (macaroon_variant == NULL)
-		return NULL;
-	g_variant_get (macaroon_variant, "(&s^as)", &macaroon, &discharges);
-
-	return snapd_auth_data_new (macaroon, discharges);
-}
-
-static SnapdClient *
-get_client (GsPlugin *plugin, GCancellable *cancellable, GError **error)
-{
-	g_autoptr(SnapdClient) client = NULL;
-	g_autoptr(SnapdAuthData) auth_data = NULL;
-
-	client = snapd_client_new ();
-	if (!snapd_client_connect_sync (client, cancellable, error))
-		return NULL;
-	auth_data = get_auth (plugin);
-	snapd_client_set_auth_data (client, auth_data);
-
-	return g_steal_pointer (&client);
-}
-
 static gboolean
 gs_plugin_snap_set_app_pixbuf_from_data (GsApp *app, const gchar *buf, gsize count, GError **error)
 {
@@ -126,7 +186,7 @@ gs_plugin_snap_set_app_pixbuf_from_data (GsApp *app, const gchar *buf, gsize cou
 	g_autoptr(GError) error_local = NULL;
 
 	loader = gdk_pixbuf_loader_new ();
-	if (!gdk_pixbuf_loader_write (loader, buf, count, &error_local)) {
+	if (!gdk_pixbuf_loader_write (loader, (const guchar *) buf, count, &error_local)) {
 		g_debug ("icon_data[%" G_GSIZE_FORMAT "]=%s", count, buf);
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
@@ -151,16 +211,14 @@ static GPtrArray *
 find_snaps (GsPlugin *plugin, SnapdFindFlags flags, const gchar *section, const gchar *query, GCancellable *cancellable, GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	g_autoptr(SnapdClient) client = NULL;
 	g_autoptr(GPtrArray) snaps = NULL;
 	guint i;
 
-	client = get_client (plugin, cancellable, error);
-	if (client == NULL)
-		return FALSE;
-	snaps = snapd_client_find_section_sync (client, flags, section, query, NULL, cancellable, NULL);
-	if (snaps == NULL)
+	snaps = snapd_client_find_section_sync (priv->client, flags, section, query, NULL, cancellable, error);
+	if (snaps == NULL) {
+		snapd_error_convert (error);
 		return NULL;
+	}
 
 	/* cache results */
 	for (i = 0; i < snaps->len; i++) {
@@ -174,19 +232,32 @@ find_snaps (GsPlugin *plugin, SnapdFindFlags flags, const gchar *section, const
 static GsApp *
 snap_to_app (GsPlugin *plugin, SnapdSnap *snap)
 {
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 	GsApp *app;
 
 	/* create a unique ID for deduplication, TODO: branch? */
 	app = gs_app_new (snapd_snap_get_name (snap));
-	gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
+	switch (snapd_snap_get_snap_type (snap)) {
+	case SNAPD_SNAP_TYPE_APP:
+		gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
+		break;
+	case SNAPD_SNAP_TYPE_KERNEL:
+	case SNAPD_SNAP_TYPE_GADGET:
+	case SNAPD_SNAP_TYPE_OS:
+		gs_app_set_kind (app, AS_APP_KIND_RUNTIME);
+		gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
+		break;
+        default:
+	case SNAPD_SNAP_TYPE_UNKNOWN:
+                break;
+	}
 	gs_app_set_scope (app, AS_APP_SCOPE_SYSTEM);
 	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_SNAP);
 	gs_app_set_management_plugin (app, "snap");
 	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_REVIEWABLE);
-	gs_app_set_name (app, GS_APP_QUALITY_HIGHEST, snapd_snap_get_name (snap));
 	if (gs_plugin_check_distro_id (plugin, "ubuntu"))
 		gs_app_add_quirk (app, AS_APP_QUIRK_PROVENANCE);
-	if (snapd_snap_get_confinement (snap) == SNAPD_CONFINEMENT_STRICT)
+	if (priv->system_confinement == SNAPD_SYSTEM_CONFINEMENT_STRICT && snapd_snap_get_confinement (snap) == SNAPD_CONFINEMENT_STRICT)
 		gs_app_add_kudo (app, GS_APP_KUDO_SANDBOXED);
 
 	return app;
@@ -224,7 +295,7 @@ gs_plugin_destroy (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_clear_object (&priv->auth);
-	g_hash_table_unref (priv->store_snaps);
+	g_clear_pointer (&priv->store_snaps, g_hash_table_unref);
 }
 
 gboolean
@@ -254,16 +325,15 @@ gs_plugin_add_installed (GsPlugin *plugin,
 			 GCancellable *cancellable,
 			 GError **error)
 {
-	g_autoptr(SnapdClient) client = NULL;
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_autoptr(GPtrArray) snaps = NULL;
 	guint i;
 
-	client = get_client (plugin, cancellable, error);
-	if (client == NULL)
-		return FALSE;
-	snaps = snapd_client_list_sync (client, cancellable, error);
-	if (snaps == NULL)
+	snaps = snapd_client_list_sync (priv->client, cancellable, error);
+	if (snaps == NULL) {
+		snapd_error_convert (error);
 		return FALSE;
+	}
 
 	for (i = 0; i < snaps->len; i++) {
 		SnapdSnap *snap = snaps->pdata[i];
@@ -304,6 +374,8 @@ gs_plugin_add_search (GsPlugin *plugin,
 static gboolean
 load_icon (GsPlugin *plugin, GsApp *app, const gchar *icon_url, GCancellable *cancellable, GError **error)
 {
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+
 	if (icon_url == NULL || g_strcmp0 (icon_url, "") == 0) {
 		g_autoptr(AsIcon) icon = as_icon_new ();
 		as_icon_set_kind (icon, AS_ICON_KIND_STOCK);
@@ -314,16 +386,13 @@ load_icon (GsPlugin *plugin, GsApp *app, const gchar *icon_url, GCancellable *ca
 
 	/* icon is optional, either loaded from snapd or from a URL */
 	if (g_str_has_prefix (icon_url, "/")) {
-		g_autoptr(SnapdClient) client = NULL;
 		g_autoptr(SnapdIcon) icon = NULL;
 
-		client = get_client (plugin, cancellable, error);
-		if (client == NULL)
-			return FALSE;
-
-		icon = snapd_client_get_icon_sync (client, gs_app_get_id (app), cancellable, error);
-		if (icon == NULL)
+		icon = snapd_client_get_icon_sync (priv->client, gs_app_get_id (app), cancellable, error);
+		if (icon == NULL) {
+			snapd_error_convert (error);
 			return FALSE;
+		}
 
 		if (!gs_plugin_snap_set_app_pixbuf_from_data (app,
 							      g_bytes_get_data (snapd_icon_get_data (icon), NULL),
@@ -333,50 +402,10 @@ load_icon (GsPlugin *plugin, GsApp *app, const gchar *icon_url, GCancellable *ca
 			return FALSE;
 		}
 	} else {
-		g_autofree gchar *cache_fn = NULL;
-		g_autoptr(SoupMessage) message = NULL;
-		g_autoptr(GdkPixbufLoader) loader = NULL;
-		g_autoptr(GError) local_error = NULL;
-
-		/* attempt to load from cache */
-		cache_fn = gs_utils_get_cache_filename ("snap-icons", icon_url, GS_UTILS_CACHE_FLAG_USE_HASH | GS_UTILS_CACHE_FLAG_WRITEABLE, error);
-		if (cache_fn == NULL)
-			return FALSE;
-		if (g_file_test (cache_fn, G_FILE_TEST_EXISTS)) {
-			g_autofree gchar *data = NULL;
-			gsize data_len;
-
-			if (g_file_get_contents (cache_fn, &data, &data_len, &local_error) &&
-			    gs_plugin_snap_set_app_pixbuf_from_data (app,
-								     data, data_len,
-								     error))
-				return TRUE;
-
-			g_warning ("Failed to load cached icon: %s", local_error->message);
-		}
-
-		/* load from URL */
-		message = soup_message_new (SOUP_METHOD_GET, icon_url);
-		if (message == NULL) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-				     "Failed to parse icon URL: %s",
-				     icon_url);
-			return FALSE;
-		}
-		soup_session_send_message (gs_plugin_get_soup_session (plugin), message);
-		if (!gs_plugin_snap_set_app_pixbuf_from_data (app,
-					(const gchar *) message->response_body->data,
-					message->response_body->length,
-					error)) {
-			g_prefix_error (error, "Failed to load %s: ", icon_url);
-			return FALSE;
-		}
-
-		/* write to cache */
-		if (!g_file_set_contents (cache_fn, message->response_body->data, message->response_body->length, &local_error))
-			g_warning ("Failed to save icon to cache: %s", local_error->message);
+		g_autoptr(AsIcon) icon = as_icon_new ();
+		as_icon_set_kind (icon, AS_ICON_KIND_REMOTE);
+		as_icon_set_url (icon, icon_url);
+		gs_app_add_icon (app, icon);
 	}
 
 	return TRUE;
@@ -401,6 +430,51 @@ get_store_snap (GsPlugin *plugin, const gchar *name, GCancellable *cancellable,
 	return g_object_ref (g_ptr_array_index (snaps, 0));
 }
 
+static void
+find_launch_app (GsApp *app, SnapdSnap *local_snap)
+{
+	const char *snap_name;
+	GPtrArray *apps;
+	guint i;
+	const char *launch_name = NULL;
+	const char *launch_desktop = NULL;
+
+	snap_name = snapd_snap_get_name (local_snap);
+	apps = snapd_snap_get_apps (local_snap);
+
+	/* Pick the "main" app from the snap.  In order of
+	 * preference, we want to pick:
+	 *
+	 *   1. the main app, provided it has a desktop file
+	 *   2. the first app with a desktop file
+	 *   3. the main app
+	 *   4. the first app
+	 *
+	 * The "main app" is one whose name matches the snap name.
+	 */
+	for (i = 0; i < apps->len; i++) {
+		SnapdApp *snap_app = apps->pdata[i];
+		const char *app_name = snapd_app_get_name (snap_app);
+		const char *app_desktop = snapd_app_get_desktop_file (snap_app);
+		gboolean is_main_app = !g_strcmp0(snap_name, app_name);
+
+		if (launch_name == NULL || is_main_app) {
+			launch_name = app_name;
+		}
+		if (launch_desktop == NULL || is_main_app) {
+			if (app_desktop != NULL) {
+				launch_desktop = app_desktop;
+			}
+		}
+	}
+
+	gs_app_set_metadata (app, "snap::launch-name", launch_name);
+	gs_app_set_metadata (app, "snap::launch-desktop", launch_desktop);
+
+	if (!launch_name)
+		gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
+}
+
 gboolean
 gs_plugin_refine_app (GsPlugin *plugin,
 		      GsApp *app,
@@ -408,7 +482,7 @@ gs_plugin_refine_app (GsPlugin *plugin,
 		      GCancellable *cancellable,
 		      GError **error)
 {
-	g_autoptr(SnapdClient) client = NULL;
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 	const gchar *id, *icon_url = NULL;
 	g_autoptr(SnapdSnap) local_snap = NULL;
 	g_autoptr(SnapdSnap) store_snap = NULL;
@@ -417,22 +491,20 @@ gs_plugin_refine_app (GsPlugin *plugin,
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "snap") != 0)
 		return TRUE;
 
-	client = get_client (plugin, cancellable, error);
-	if (client == NULL)
-		return FALSE;
 	id = gs_app_get_id (app);
 	if (id == NULL)
 		id = gs_app_get_source_default (app);
 
 	/* get information from installed snaps */
-	local_snap = snapd_client_list_one_sync (client, id, cancellable, NULL);
+	local_snap = snapd_client_list_one_sync (priv->client, id, cancellable, NULL);
 	if (local_snap != NULL) {
-		GPtrArray *apps;
-		const gchar *launch_name = NULL;
-
+		const gchar *name;
 		if (gs_app_get_state (app) == AS_APP_STATE_UNKNOWN)
 			gs_app_set_state (app, AS_APP_STATE_INSTALLED);
-		gs_app_set_name (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_name (local_snap));
+		name = snapd_snap_get_title (local_snap);
+		if (name == NULL || g_strcmp0 (name, "") == 0)
+			name = snapd_snap_get_name (local_snap);
+		gs_app_set_name (app, GS_APP_QUALITY_NORMAL, name);
 		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_summary (local_snap));
 		gs_app_set_description (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_description (local_snap));
 		gs_app_set_version (app, snapd_snap_get_version (local_snap));
@@ -443,25 +515,22 @@ gs_plugin_refine_app (GsPlugin *plugin,
 		if (g_strcmp0 (icon_url, "") == 0)
 			icon_url = NULL;
 
-		apps = snapd_snap_get_apps (local_snap);
-		if (apps->len > 0)
-			launch_name = snapd_app_get_name (apps->pdata[0]);
-		if (launch_name != NULL)
-			gs_app_set_metadata (app, "snap::launch-name", launch_name);
-		else
-			gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
+		find_launch_app (app, local_snap);
 	}
 
 	/* get information from snap store */
 	store_snap = get_store_snap (plugin, id, cancellable, NULL);
 	if (store_snap != NULL) {
 		GPtrArray *screenshots;
-		const gchar *screenshot_url = NULL;
+		const gchar *name, *screenshot_url = NULL;
 
 		if (gs_app_get_state (app) == AS_APP_STATE_UNKNOWN)
 			gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
 
-		gs_app_set_name (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_name (store_snap));
+		name = snapd_snap_get_title (store_snap);
+		if (name == NULL || g_strcmp0 (name, "") == 0)
+			name = snapd_snap_get_name (store_snap);
+		gs_app_set_name (app, GS_APP_QUALITY_NORMAL, name);
 		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_summary (store_snap));
 		gs_app_set_description (app, GS_APP_QUALITY_NORMAL, snapd_snap_get_description (store_snap));
 		gs_app_set_version (app, snapd_snap_get_version (store_snap));
@@ -509,8 +578,10 @@ gs_plugin_refine_app (GsPlugin *plugin,
 
 	/* load icon if requested */
 	if (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON) {
-		if (!load_icon (plugin, app, icon_url, cancellable, error))
+		if (!load_icon (plugin, app, icon_url, cancellable, error)) {
+			snapd_error_convert (error);
 			return FALSE;
+		}
 	}
 
 	return TRUE;
@@ -540,18 +611,16 @@ gs_plugin_app_install (GsPlugin *plugin,
 		       GCancellable *cancellable,
 		       GError **error)
 {
-	g_autoptr(SnapdClient) client = NULL;
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 
 	/* We can only install apps we know of */
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "snap") != 0)
 		return TRUE;
 
 	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
-	client = get_client (plugin, cancellable, error);
-	if (client == NULL)
-		return FALSE;
-	if (!snapd_client_install_sync (client, gs_app_get_id (app), NULL, progress_cb, app, cancellable, error)) {
+	if (!snapd_client_install2_sync (priv->client, SNAPD_INSTALL_FLAGS_NONE, gs_app_get_id (app), NULL, NULL, progress_cb, app, cancellable, error)) {
 		gs_app_set_state_recover (app);
+		snapd_error_convert (error);
 		return FALSE;
 	}
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
@@ -564,17 +633,12 @@ gs_plugin_app_install (GsPlugin *plugin,
 static gboolean
 is_graphical (GsPlugin *plugin, GsApp *app, GCancellable *cancellable)
 {
-	g_autoptr(SnapdClient) client = NULL;
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_autoptr(GPtrArray) plugs = NULL;
 	guint i;
 	g_autoptr(GError) error = NULL;
 
-	client = get_client (plugin, cancellable, &error);
-	if (client == NULL) {
-		g_warning ("Failed to make snapd client: %s", error->message);
-		return FALSE;
-	}
-	if (!snapd_client_get_interfaces_sync (client, &plugs, NULL, cancellable, &error)) {
+	if (!snapd_client_get_interfaces_sync (priv->client, &plugs, NULL, cancellable, &error)) {
 		g_warning ("Failed to check interfaces: %s", error->message);
 		return FALSE;
 	}
@@ -605,8 +669,7 @@ gs_plugin_launch (GsPlugin *plugin,
 		  GError **error)
 {
 	const gchar *launch_name;
-	g_autofree gchar *binary_name = NULL;
-	GAppInfoCreateFlags flags = G_APP_INFO_CREATE_NONE;
+	const gchar *launch_desktop;
 	g_autoptr(GAppInfo) info = NULL;
 
 	/* We can only launch apps we know of */
@@ -614,17 +677,26 @@ gs_plugin_launch (GsPlugin *plugin,
 		return TRUE;
 
 	launch_name = gs_app_get_metadata_item (app, "snap::launch-name");
+	launch_desktop = gs_app_get_metadata_item (app, "snap::launch-desktop");
 	if (!launch_name)
 		return TRUE;
 
-	if (g_strcmp0 (launch_name, gs_app_get_id (app)) == 0)
-		binary_name = g_strdup_printf ("/snap/bin/%s", launch_name);
-	else
-		binary_name = g_strdup_printf ("/snap/bin/%s.%s", gs_app_get_id (app), launch_name);
+	if (launch_desktop) {
+		info = (GAppInfo *)g_desktop_app_info_new_from_filename (launch_desktop);
+	} else {
+		g_autofree gchar *binary_name = NULL;
+		GAppInfoCreateFlags flags = G_APP_INFO_CREATE_NONE;
+
+		if (g_strcmp0 (launch_name, gs_app_get_id (app)) == 0)
+			binary_name = g_strdup_printf ("/snap/bin/%s", launch_name);
+		else
+			binary_name = g_strdup_printf ("/snap/bin/%s.%s", gs_app_get_id (app), launch_name);
+
+		if (!is_graphical (plugin, app, cancellable))
+			flags |= G_APP_INFO_CREATE_NEEDS_TERMINAL;
+		info = g_app_info_create_from_commandline (binary_name, NULL, flags, error);
+	}
 
-	if (!is_graphical (plugin, app, cancellable))
-		flags |= G_APP_INFO_CREATE_NEEDS_TERMINAL;
-	info = g_app_info_create_from_commandline (binary_name, NULL, flags, error);
 	if (info == NULL)
 		return FALSE;
 
@@ -637,18 +709,16 @@ gs_plugin_app_remove (GsPlugin *plugin,
 		      GCancellable *cancellable,
 		      GError **error)
 {
-	g_autoptr(SnapdClient) client = NULL;
+	GsPluginData *priv = gs_plugin_get_data (plugin);
 
 	/* We can only remove apps we know of */
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "snap") != 0)
 		return TRUE;
 
 	gs_app_set_state (app, AS_APP_STATE_REMOVING);
-	client = get_client (plugin, cancellable, error);
-	if (client == NULL)
-		return FALSE;
-	if (!snapd_client_remove_sync (client, gs_app_get_id (app), progress_cb, app, cancellable, error)) {
+	if (!snapd_client_remove_sync (priv->client, gs_app_get_id (app), progress_cb, app, cancellable, error)) {
 		gs_app_set_state_recover (app);
+		snapd_error_convert (error);
 		return FALSE;
 	}
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
@@ -663,34 +733,18 @@ gs_plugin_auth_login (GsPlugin *plugin, GsAuth *auth,
 	g_autoptr(SnapdAuthData) auth_data = NULL;
 	g_autoptr(GVariant) macaroon_variant = NULL;
 	g_autofree gchar *serialized_macaroon = NULL;
-	g_autoptr(GError) local_error = NULL;
 
 	if (auth != priv->auth)
 		return TRUE;
 
-	auth_data = snapd_login_sync (gs_auth_get_username (auth), gs_auth_get_password (auth), gs_auth_get_pin (auth), NULL, &local_error);
+	auth_data = snapd_login_sync (gs_auth_get_username (auth), gs_auth_get_password (auth), gs_auth_get_pin (auth), NULL, error);
 	if (auth_data == NULL) {
-		if (g_error_matches (local_error, SNAPD_ERROR, SNAPD_ERROR_TWO_FACTOR_REQUIRED)) {
-			g_set_error_literal (error,
-					     GS_PLUGIN_ERROR,
-					     GS_PLUGIN_ERROR_PIN_REQUIRED,
-					     local_error->message);
-		} else if (g_error_matches (local_error, SNAPD_ERROR, SNAPD_ERROR_AUTH_DATA_INVALID) ||
-			   g_error_matches (local_error, SNAPD_ERROR, SNAPD_ERROR_TWO_FACTOR_INVALID)) {
-			g_set_error_literal (error,
-					     GS_PLUGIN_ERROR,
-					     GS_PLUGIN_ERROR_AUTH_INVALID,
-					     local_error->message);
-		} else {
-			g_dbus_error_strip_remote_error (local_error);
-			g_set_error_literal (error,
-					     GS_PLUGIN_ERROR,
-					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
-					     local_error->message);
-		}
+		snapd_error_convert (error);
 		return FALSE;
 	}
 
+	snapd_client_set_auth_data (priv->client, auth_data);
+
 	macaroon_variant = g_variant_new ("(s^as)",
 					  snapd_auth_data_get_macaroon (auth_data),
 					  snapd_auth_data_get_discharges (auth_data));
