@@ -440,7 +440,7 @@ gs_details_page_refresh_progress (GsDetailsPage *self)
 	switch (state) {
 	case AS_APP_STATE_INSTALLING:
 		percentage = gs_app_get_progress (self->app);
-		if (percentage > 0) {
+		if (percentage <= 100) {
 			g_autofree gchar *str = g_strdup_printf ("%u%%", percentage);
 			gtk_label_set_label (GTK_LABEL (self->label_progress_percentage), str);
 			gtk_widget_set_visible (self->label_progress_percentage, TRUE);
@@ -595,6 +595,7 @@ gs_details_page_refresh_screenshots (GsDetailsPage *self)
 	case AS_APP_KIND_DRIVER:
 	case AS_APP_KIND_INPUT_METHOD:
 	case AS_APP_KIND_LOCALIZATION:
+	case AS_APP_KIND_RUNTIME:
 		gtk_widget_set_visible (self->box_details_screenshot_fallback, FALSE);
 		break;
 	default:
@@ -1533,6 +1534,8 @@ gs_details_page_file_to_app_cb (GObject *source,
 						    &error);
 	if (list == NULL) {
 		g_warning ("failed to convert file to GsApp: %s", error->message);
+		/* go back to the overview */
+		gs_shell_change_mode (self->shell, GS_SHELL_MODE_OVERVIEW, NULL, FALSE);
 	} else {
 		GsApp *app = gs_app_list_index (list, 0);
 		set_app (self, app);
@@ -1554,6 +1557,8 @@ gs_details_page_url_to_app_cb (GObject *source,
 						    &error);
 	if (list == NULL) {
 		g_warning ("failed to convert URL to GsApp: %s", error->message);
+		/* go back to the overview */
+		gs_shell_change_mode (self->shell, GS_SHELL_MODE_OVERVIEW, NULL, FALSE);
 	} else {
 		GsApp *app = gs_app_list_index (list, 0);
 		set_app (self, app);
