@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2017 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -78,6 +78,7 @@ typedef struct
 	gchar			*events_info_uri;
 	gboolean		 profile_mode;
 	gboolean		 in_mode_change;
+	gboolean		 show_detailed_error;
 	GsPage			*page_last;
 } GsShellPrivate;
 
@@ -900,6 +901,8 @@ gs_shell_show_event_refresh (GsShell *shell, GsPluginEvent *event)
 		g_string_append (str, _("Unable to download updates: you do not have"
 				        " permission to install software"));
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
 		/* TRANSLATORS: failure text for the in-app notification */
 		g_string_append (str, _("Unable to get list of updates"));
@@ -918,6 +921,10 @@ gs_shell_show_event_refresh (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1049,6 +1056,8 @@ gs_shell_show_event_install (GsShell *shell, GsPluginEvent *event)
 					       "AC power is required"),
 					str_app);
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
 		/* TRANSLATORS: failure text for the in-app notification,
 		 * where the %s is the application name (e.g. "GIMP") */
@@ -1068,6 +1077,10 @@ gs_shell_show_event_install (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1149,6 +1162,8 @@ gs_shell_show_event_update (GsShell *shell, GsPluginEvent *event)
 					       "AC power is required"),
 					str_app);
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
 		/* TRANSLATORS: failure text for the in-app notification,
 		 * where the %s is the application name (e.g. "GIMP") */
@@ -1168,6 +1183,10 @@ gs_shell_show_event_update (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1247,6 +1266,8 @@ gs_shell_show_event_upgrade (GsShell *shell, GsPluginEvent *event)
 					       "AC power is required"),
 					str_app);
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
 		/* TRANSLATORS: failure text for the in-app notification,
 		 * where the %s is the distro name (e.g. "Fedora 25") */
@@ -1266,6 +1287,10 @@ gs_shell_show_event_upgrade (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1311,6 +1336,8 @@ gs_shell_show_event_remove (GsShell *shell, GsPluginEvent *event)
 					       "AC power is required"),
 					str_app);
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
 		/* TRANSLATORS: failure text for the in-app notification,
 		 * where the %s is the application name (e.g. "GIMP") */
@@ -1330,6 +1357,10 @@ gs_shell_show_event_remove (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1367,7 +1398,11 @@ gs_shell_show_event_launch (GsShell *shell, GsPluginEvent *event)
 					"and try again"));
 		buttons |= GS_SHELL_EVENT_BUTTON_NO_SPACE;
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
+		/* TRANSLATORS: we failed to get a proper error code */
+		g_string_append (str, _("Sorry, something went wrong"));
 		break;
 	}
 	if (str->len == 0)
@@ -1383,6 +1418,10 @@ gs_shell_show_event_launch (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1392,6 +1431,7 @@ static gboolean
 gs_shell_show_event_file_to_app (GsShell *shell, GsPluginEvent *event)
 {
 	GsShellEventButtons buttons = GS_SHELL_EVENT_BUTTON_NONE;
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 	const GError *error = gs_plugin_event_get_error (event);
 	g_autoptr(GString) str = g_string_new (NULL);
 
@@ -1407,12 +1447,20 @@ gs_shell_show_event_file_to_app (GsShell *shell, GsPluginEvent *event)
 					"and try again"));
 		buttons |= GS_SHELL_EVENT_BUTTON_NO_SPACE;
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
+		/* TRANSLATORS: we failed to get a proper error code */
+		g_string_append (str, _("Sorry, something went wrong"));
 		break;
 	}
 	if (str->len == 0)
 		return FALSE;
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1466,7 +1514,11 @@ gs_shell_show_event_fallback (GsShell *shell, GsPluginEvent *event)
 		/* TRANSLATORS: need to be connected to the AC power */
 		g_string_append (str, _("AC power is required"));
 		break;
+	case GS_PLUGIN_ERROR_CANCELLED:
+		break;
 	default:
+		/* TRANSLATORS: we failed to get a proper error code */
+		g_string_append (str, _("Sorry, something went wrong"));
 		break;
 	}
 	if (str->len == 0)
@@ -1482,6 +1534,10 @@ gs_shell_show_event_fallback (GsShell *shell, GsPluginEvent *event)
 		}
 	}
 
+	/* add extra debugging for debug builds */
+	if (priv->show_detailed_error)
+		g_string_append_printf (str, "\n%s", error->message);
+
 	/* show in-app notification */
 	gs_shell_show_event_app_notify (shell, str->str, buttons);
 	return TRUE;
@@ -1498,6 +1554,13 @@ gs_shell_show_event (GsShell *shell, GsPluginEvent *event)
 	if (error == NULL)
 		return FALSE;
 
+	/* name and shame the plugin */
+	if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_TIMED_OUT)) {
+		gs_shell_show_event_app_notify (shell, error->message,
+						GS_SHELL_EVENT_BUTTON_NONE);
+		return TRUE;
+	}
+
 	/* split up the events by action */
 	action = gs_plugin_event_get_action (event);
 	switch (action) {
@@ -1956,6 +2019,7 @@ gs_shell_init (GsShell *shell)
 	priv->pages = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	priv->back_entry_stack = g_queue_new ();
 	priv->ignore_primary_buttons = FALSE;
+	priv->show_detailed_error = TRUE;
 	priv->modal_dialogs = g_ptr_array_new_with_free_func ((GDestroyNotify) gtk_widget_destroy);
 }
 
